Package: RCAS
Type: Package
Title: RNA Centric Annotation System
Version: 1.36.0
Date: 2024-03-19
Authors@R: c(person("Bora", "Uyar", email='bora.uyar@mdc-berlin.de', role=c("aut", "cre")),
              person("Dilmurat", "Yusuf", role=c("aut")),
              person("Ricardo", "Wurmus", role=c("aut")),
              person("Altuna", "Akalin", role=c("aut")))
Description: RCAS is an R/Bioconductor package designed as a generic reporting tool for the
    functional analysis of transcriptome-wide regions of interest detected by
    high-throughput experiments. Such transcriptomic regions could be, for instance,
    signal peaks detected by CLIP-Seq analysis for protein-RNA interaction sites,
    RNA modification sites (alias the epitranscriptome), CAGE-tag locations, or any
    other collection of query regions at the level of the transcriptome. RCAS
    produces in-depth annotation summaries and coverage profiles based on the
    distribution of the query regions with respect to transcript features (exons,
    introns, 5'/3' UTR regions, exon-intron boundaries, promoter regions). Moreover,
    RCAS can carry out functional enrichment analyses and discriminative motif
    discovery. 
License: Artistic-2.0
LazyData: TRUE
Depends: R (>= 3.5.0), plotly (>= 4.5.2), DT (>= 0.2), data.table
Imports: GenomicRanges, IRanges, BSgenome, BSgenome.Hsapiens.UCSC.hg19,
        GenomeInfoDb (>= 1.12.0), Biostrings, rtracklayer,
        GenomicFeatures, txdbmaker, rmarkdown (>= 0.9.5), genomation
        (>= 1.5.5), knitr (>= 1.12.3), BiocGenerics, S4Vectors,
        plotrix, pbapply, RSQLite, proxy, pheatmap, ggplot2, cowplot,
        seqLogo, utils, ranger, gprofiler2
RoxygenNote: 7.3.1
Suggests: testthat, covr, BiocManager
SystemRequirements: pandoc (>= 1.12.3)
VignetteBuilder: knitr
biocViews: Software, GeneTarget, MotifAnnotation, MotifDiscovery, GO,
        Transcriptomics, GenomeAnnotation, GeneSetEnrichment, Coverage
git_url: https://git.bioconductor.org/packages/RCAS
git_branch: RELEASE_3_22
git_last_commit: 780ed18
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:59:51 UTC; biocbuild
Author: Bora Uyar [aut, cre],
  Dilmurat Yusuf [aut],
  Ricardo Wurmus [aut],
  Altuna Akalin [aut]
Maintainer: Bora Uyar <bora.uyar@mdc-berlin.de>
