# Generated by roxygen2: do not edit by hand

export(calculateCoverageProfile)
export(calculateCoverageProfileFromTxdb)
export(calculateCoverageProfileList)
export(calculateCoverageProfileListFromTxdb)
export(checkSeqDb)
export(createControlRegions)
export(createDB)
export(createOrthologousGeneSetList)
export(deleteSampleDataFromDB)
export(discoverFeatureSpecificMotifs)
export(extractSequences)
export(findDifferentialMotifs)
export(findEnrichedFunctions)
export(generateKmers)
export(getFeatureBoundaryCoverage)
export(getFeatureBoundaryCoverageBin)
export(getFeatureBoundaryCoverageMulti)
export(getIntervalOverlapMatrix)
export(getMotifSummaryTable)
export(getPWM)
export(getTargetedGenesTable)
export(getTxdbFeatures)
export(getTxdbFeaturesFromGRanges)
export(importBed)
export(importBedFiles)
export(importGtf)
export(parseMsigdb)
export(plotFeatureBoundaryCoverage)
export(printMsigdbDataset)
export(queryGff)
export(retrieveOrthologs)
export(runGSEA)
export(runMotifDiscovery)
export(runMotifRG)
export(runReport)
export(runReportMetaAnalysis)
export(runTopGO)
export(summarizeDatabaseContent)
export(summarizeQueryRegions)
export(summarizeQueryRegionsMulti)
import(DT)
import(GenomicRanges)
import(RSQLite)
import(ggplot2)
import(knitr)
import(ranger)
import(rmarkdown)
importFrom(BSgenome,available.genomes)
importFrom(BSgenome.Hsapiens.UCSC.hg19,Hsapiens)
importFrom(BiocGenerics,unlist)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,vmatchPattern)
importFrom(GenomeInfoDb,keepStandardChromosomes)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicFeatures,cdsBy)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,fiveUTRsByTranscript)
importFrom(GenomicFeatures,intronsByTranscript)
importFrom(GenomicFeatures,promoters)
importFrom(GenomicFeatures,threeUTRsByTranscript)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicRanges,GRangesList)
importFrom(IRanges,width)
importFrom(RSQLite,SQLite)
importFrom(RSQLite,dbConnect)
importFrom(RSQLite,dbDisconnect)
importFrom(RSQLite,dbWriteTable)
importFrom(S4Vectors,Reduce)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(cowplot,plot_grid)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,setkey)
importFrom(genomation,ScoreMatrix)
importFrom(genomation,ScoreMatrixBin)
importFrom(gprofiler2,gost)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(pbapply,pblapply)
importFrom(pbapply,pbsapply)
importFrom(pheatmap,pheatmap)
importFrom(plotly,add_ribbons)
importFrom(plotly,add_trace)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotrix,std.error)
importFrom(proxy,dist)
importFrom(rtracklayer,import.bed)
importFrom(rtracklayer,import.gff)
importFrom(seqLogo,seqLogo)
importFrom(stats,fisher.test)
importFrom(txdbmaker,makeTxDbFromGRanges)
importFrom(utils,read.table)
