% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{runTopGO}
\alias{runTopGO}
\title{runTopGO}
\usage{
runTopGO()
}
\description{
This function is deprecated. Use findEnrichedFunctions instead.
}
