% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{runGSEA}
\alias{runGSEA}
\title{runGSEA}
\usage{
runGSEA()
}
\description{
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.
}
