% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{runMotifRG}
\alias{runMotifRG}
\title{run motifRG}
\usage{
runMotifRG()
}
\description{
run motifRG
}
