% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{getTxdbFeatures}
\alias{getTxdbFeatures}
\title{getTxdbFeatures}
\usage{
getTxdbFeatures()
}
\description{
This function is deprecated. Use getTxdbFeaturesFromGRanges instead.
}
